import { world, system, ItemStack } from "@minecraft/server";

// Listen for when a player interacts with a jar block
world.afterEvents.itemStartUseOn.subscribe(handleJarInteraction);
world.beforeEvents.playerBreakBlock.subscribe(handleJarBreak);


// Custom Block Component: "paragonia_bm:jar"
world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.blockComponentRegistry.registerCustomComponent("paragonia_bm:jar", {
        onPlace: handleButterflyJarPlacement
    });
});

// Custom Item Component: "paragonia_bm:butterfly_item"
world.beforeEvents.worldInitialize.subscribe(initEvent => {
    initEvent.itemComponentRegistry.registerCustomComponent("paragonia_bm:butterfly_item", {
        onUseOn: handleButterflyItemUse
    });
});

// Mapping cardinal directions to rotation values
const directionToRotation = {
  north: 180,
  south: 0,
  east: -90,
  west: 90,
};

// Map items to their corresponding variant value (Add butterfly entity mappings to this const)
const butterflyVariants = {
    "paragonia_bm:item_butterfly_red": 1,
    "paragonia_bm:item_butterfly_orange": 2,
    "paragonia_bm:item_butterfly_yellow": 3,
    "paragonia_bm:item_butterfly_lime": 4,
    "paragonia_bm:item_butterfly_green": 5,
    "paragonia_bm:item_butterfly_cyan": 6,
    "paragonia_bm:item_butterfly_lightblue": 7,
    "paragonia_bm:item_butterfly_blue": 8,
    "paragonia_bm:item_butterfly_purple": 9,
    "paragonia_bm:item_butterfly_magenta": 10,
    "paragonia_bm:item_butterfly_pink": 11,
    "paragonia_bm:item_butterfly_brown": 12,
    "paragonia_bm:item_butterfly_white": 13,
    "paragonia_bm:item_butterfly_lightgray": 14,
    "paragonia_bm:item_butterfly_gray": 15,
    "paragonia_bm:item_butterfly_black": 16
};

// Reverse mapping for item spawning from jars
const variantToItem = Object.fromEntries(Object.entries(butterflyVariants).map(([item, variant]) => [variant, item]));

// Map items to their corresponding butterfly entity types
const butterflyEntities = Object.fromEntries(
    Object.entries(butterflyVariants).map(([item, variant]) => [item, item.replace("item_", "")])
);

// Handles placement of the Butterfly Jar block
function handleButterflyJarPlacement(event) {
    let { block } = event;
    let dimension = block.dimension;
    let x = block.location.x + 0.5;
    let y = block.location.y;
    let z = block.location.z + 0.5;
    let blockDirection = block.permutation.getState("minecraft:cardinal_direction");
    let yRotation = directionToRotation[blockDirection] ?? 0;

    try {
        dimension.runCommandAsync(`summon paragonia_bm:butterfly_jar ${x} ${y} ${z} ${yRotation} 0`);
    } catch (error) {
        console.warn(`Failed to spawn butterfly jar entity: ${error}`);
    }
}

// Handles player interaction on jars
function handleJarInteraction(event) {
    const { source: player, block, itemStack } = event;
    if (!block || block.typeId !== "paragonia_bm:butterfly_jar") return;

    let dimension = block.dimension;
    let x = block.location.x + 0.5;
    let y = block.location.y + 0.5;
    let z = block.location.z + 0.5;

    system.run(() => {
        for (let entity of dimension.getEntitiesAtBlockLocation(block.location)) {
            if (entity.typeId === "paragonia_bm:butterfly_jar") {
                let currentVariant = entity.getProperty("paragonia:variant");
                
                // If interacting with an empty hand, remove butterfly
                if (!itemStack && currentVariant > 0) {
                    let butterflyItem = variantToItem[currentVariant];
                    if (butterflyItem) {
                        console.log(`Removing butterfly ${butterflyItem} from jar.`);
                        let itemStack = new ItemStack(butterflyItem, 1);
                        dimension.spawnItem(itemStack, { x, y, z });
                        entity.setProperty("paragonia:variant", 0); // Reset jar after removal
                        console.log("Butterfly removed, jar reset.");
                    }
                }
                break;
            }
        }
    });
}

// Handles using a butterfly item on a block
function handleButterflyItemUse(event) {
    const { source: player, block, itemStack } = event;
    let dimension = block.dimension;
    let x = block.location.x + 0.5;
    let y = block.location.y + 0.5;
    let z = block.location.z + 0.5;
    
    if (block.typeId === "paragonia_bm:butterfly_jar") {
        system.run(() => {
            for (let entity of dimension.getEntitiesAtBlockLocation(block.location)) {
                if (entity.typeId === "paragonia_bm:butterfly_jar") {
                    let currentVariant = entity.getProperty("paragonia:variant");
                    if (itemStack && butterflyVariants[itemStack.typeId]) {
                        if (currentVariant > 0) return;
                        entity.setProperty("paragonia:variant", butterflyVariants[itemStack.typeId]);
                        player.runCommandAsync(`clear @s ${itemStack.typeId} 0 1`);
                    }
                    break;
                }
            }
        });
    } else if (itemStack && butterflyVariants[itemStack.typeId]) {
        dimension.runCommandAsync(`summon ${butterflyEntities[itemStack.typeId]} ${x} ${y} ${z}`);
        player.runCommandAsync(`clear @s ${itemStack.typeId} 0 1`);
    }
}

// Handles breaking a Butterfly Jar block
function handleJarBreak(event) {
    const { block, dimension } = event;
    if (block.typeId !== "paragonia_bm:butterfly_jar") return;

    let x = block.location.x + 0.5;
    let y = block.location.y + 0.5;
    let z = block.location.z + 0.5;

    system.run(() => {
        for (let entity of dimension.getEntitiesAtBlockLocation(block.location)) {
            if (entity.typeId === "paragonia_bm:butterfly_jar") {
                let currentVariant = entity.getProperty("paragonia:variant");
                if (currentVariant > 0) {
                    let butterflyItem = variantToItem[currentVariant];
                    if (butterflyItem) {
                        let itemStack = new ItemStack(butterflyItem, 1);
                        dimension.spawnItem(itemStack, { x, y, z });
                    }
                }
                entity.remove();
                break;
            }
        }
    });
}
